@@ -1,8 +1,9 @@
 #[cfg(feature = "js-esbuild")]
 use {
     crate::minify::esbuild::minify_using_esbuild,
-    aho_corasick::{AhoCorasick, AhoCorasickBuilder},
-    esbuild_rs::{Loader, TransformOptions, TransformOptionsBuilder},
+    esbuild_rs::{
+        Charset, LegalComments, Loader, SourceMap, TransformOptions, TransformOptionsBuilder,
+    },
     lazy_static::lazy_static,
     std::sync::Arc,
 };
@@ -11,15 +12,15 @@ use crate::cfg::Cfg;
 
 #[cfg(feature = "js-esbuild")]
 lazy_static! {
-    static ref STYLE_END: AhoCorasick = AhoCorasickBuilder::new()
-        .ascii_case_insensitive(true)
-        .build(&["</style"]);
     pub static ref MINIFY_CSS_TRANSFORM_OPTIONS: Arc<TransformOptions> = {
         let mut builder = TransformOptionsBuilder::new();
+        builder.charset = Charset::UTF8;
+        builder.legal_comments = LegalComments::None;
         builder.loader = Loader::CSS;
         builder.minify_identifiers = true;
         builder.minify_syntax = true;
         builder.minify_whitespace = true;
+        builder.source_map = SourceMap::None;
         builder.build()
     };
 }
@@ -34,11 +35,6 @@ pub fn minify_css(cfg: &Cfg, out: &mut Vec<u8>, code: &[u8]) {
     if !cfg.minify_css {
         out.extend_from_slice(&code);
     } else {
-        minify_using_esbuild(
-            out,
-            code,
-            &MINIFY_CSS_TRANSFORM_OPTIONS.clone(),
-            Some(&STYLE_END),
-        );
+        minify_using_esbuild(out, code, &MINIFY_CSS_TRANSFORM_OPTIONS.clone());
     }
 }
